use super::{Node, NodeEnum, Precedence, constant::Constant};

#[derive(Clone, Debug, PartialEq)]
pub struct Add {
    left: Box<NodeEnum>,
    right: Box<NodeEnum>,
}

impl Node for Add {
    fn evaluate(&self, env: &mut super::Environment) -> Result<super::NodeEnum, String> {
        let evaluated_left = self.left.evaluate(env)?;
        let evaluated_right = self.right.evaluate(env)?;

        match (evaluated_left, evaluated_right) {
            (NodeEnum::Constant(a), NodeEnum::Constant(b)) => {
                Ok(Constant::new(a.get_value() + b.get_value()).into())
            }
            _ => Err(format!("Invalid Add operation: {:?}", self)),
        }
    }

    fn to_string(&self) -> String {
        let left_string = if self.left.precedence() <= self.precedence() {
            format!("({})", self.left.to_string())
        } else {
            self.left.to_string()
        };
        let right_string = if self.right.precedence() <= self.precedence() {
            format!("({})", self.right.to_string())
        } else {
            self.right.to_string()
        };
        format!("{}+{}", left_string, right_string)
    }

    fn precedence(&self) -> Precedence {
        Precedence::Term
    }
}

impl Add {
    pub fn new(left: NodeEnum, right: NodeEnum) -> Self {
        Self {
            left: Box::new(left),
            right: Box::new(right),
        }
    }
}
