use std::fmt::Display;

use super::{Node, NodeEnum, Precedence};

#[derive(Clone, Debug, PartialEq)]
pub struct Constant {
    value: f64,
}

impl Node for Constant {
    fn evaluate(&self, _: &mut super::Environment) -> Result<super::NodeEnum, String> {
        Ok(self.clone().into())
    }

    fn to_string(&self) -> String {
        self.value.to_string()
    }

    fn precedence(&self) -> Precedence {
        Precedence::Primary
    }
}

impl Constant {
    pub fn new(value: f64) -> Self {
        Self { value }
    }

    pub fn get_value(&self) -> f64 {
        self.value
    }

    pub fn set_value(&mut self, value: f64) {
        self.value = value;
    }
}
