use std::{cmp::Ordering, collections::HashMap, fmt::Display};

use add::Add;
use constant::Constant;
use enum_dispatch::enum_dispatch;
use subtract::Subtract;

pub mod constant;
pub mod add;
pub mod subtract;

#[enum_dispatch]
#[enum_dispatch(Debug, Clone, PartialEq, PartialOrd, ToString)]
#[derive(Debug, Clone, PartialEq)]
pub enum NodeEnum {
    Constant(Constant),
    Add(Add),
    Subtract(Subtract)
}

#[derive(PartialEq, Eq, PartialOrd, Ord)]
pub enum Precedence {
    Term,
    Factor,
    Primary,
}

pub type Environment = HashMap<u16, NodeEnum>;

#[enum_dispatch(NodeEnum)]
pub trait Node {
    fn evaluate(&self, _: &mut Environment) -> Result<NodeEnum, String>;
    fn to_string(&self) -> String;
    fn precedence(&self) -> Precedence;
}

impl Display for NodeEnum {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        write!(f, "{}", Node::to_string(self))
    }
}
